"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @export
 * @class Getsy
 */
var jqueryProxy = require("jquery");
var utils_1 = require("./utils");
var jquery = jqueryProxy.default || jqueryProxy;
var defaults = {
    corsProxy: 'https://crossorigin.me/',
    resolveURLs: true,
    iframe: false,
};
var Getsy = (function () {
    function Getsy(url, onLoad, _a) {
        var _b = _a === void 0 ? defaults : _a, _c = _b.corsProxy, corsProxy = _c === void 0 ? defaults.corsProxy : _c, _d = _b.resolveURLs, resolveURLs = _d === void 0 ? defaults.resolveURLs : _d, _e = _b.iframe, iframe = _e === void 0 ? defaults.iframe : _e;
        var _this = this;
        this.url = url;
        this.corsProxy = corsProxy;
        jquery.get(utils_1.addTrailingSlash(corsProxy) + url, 'html').done(function (data) {
            _this.content = resolveURLs ? utils_1.resolveRelativeURLs(data, url) : data;
            if (iframe) {
                var _a = iframe, _b = _a.width, width = _b === void 0 ? '100%' : _b, _c = _a.height, height = _c === void 0 ? '100%' : _c;
                // Make iframe script:
                var iframeScript = document.createElement('script');
                iframeScript.textContent = 'document.open();document.write(' + JSON.stringify(_this.content) + ');document.close()';
                // Make and append iframe element:
                var iframeEl = document.createElement('iframe');
                iframeEl.className = 'getsy-frame';
                iframeEl.style.width = width;
                iframeEl.style.height = height;
                iframeEl.style.position = 'fixed';
                iframeEl.style.opacity = '0';
                iframeEl.style.zIndex = '-1000';
                iframeEl.style.pointerEvents = 'none';
                document.body.appendChild(iframeEl);
                // Append iframe script to iframe element:
                var iframeDoc = iframeEl.contentDocument || iframeEl.contentWindow.document;
                iframeDoc.body.appendChild(iframeScript);
                // Save a reference to the iframe and doc.
                _this.iframe = iframeEl;
                _this.iframeDoc = iframeDoc;
                return iframeEl.contentWindow.onload = function () { return onLoad(null, _this); };
            }
            return onLoad(null, _this);
        }).fail(function () { return onLoad(Error('Failed to load site.')); });
    }
    Getsy.prototype.getMe = function (sel) {
        return jquery(sel, this.iframeDoc || this.content);
    };
    Getsy.prototype.scroll = function (numberOfTimes, element, interval, retries) {
        var _this = this;
        if (element === void 0) { element = this.getMe('body')[0]; }
        if (interval === void 0) { interval = 2000; }
        if (retries === void 0) { retries = 5; }
        return new Promise(function (resolve, reject) {
            if (!_this.iframe)
                return reject(Error('Scroll can only be used in iframe mode.'));
            if (numberOfTimes < 1)
                return reject(Error('Number of times is less than 1.'));
            var times = numberOfTimes; // Keep track of times.
            var totalRetries = 0; // Keep track of how many retries per time.
            var initialHeight = element.scrollHeight; // Save initialHeight for resolve message.
            var lastHeight = initialHeight; // Keep track of last height.
            var tries = 0; // Keep track of retries.
            element.scrollTop = initialHeight; // Scroll down.
            setTimeout(function infiniteScroll() {
                if (lastHeight < element.scrollHeight) {
                    lastHeight = element.scrollHeight;
                    element.scrollTop = lastHeight;
                    totalRetries += tries;
                    times -= 1;
                    if (times <= 0)
                        return resolve({ succesfulTimes: numberOfTimes - times, totalRetries: totalRetries });
                    tries = 0;
                    return setTimeout(infiniteScroll, interval); // Get more content.
                }
                // No new content yet:
                element.scrollTop = element.scrollHeight; // Try scrolling again.
                tries += 1;
                if (tries >= retries)
                    return resolve({ succesfulTimes: numberOfTimes - times, totalRetries: totalRetries });
                setTimeout(infiniteScroll, interval); // Retry
            }, interval);
        });
    };
    Getsy.prototype.hideFrame = function () {
        if (!this.iframe)
            throw Error('No iframe to hide.');
        this.iframe.style.opacity = '0';
        this.iframe.style.zIndex = '-1000';
        this.iframe.style.pointerEvents = 'none';
    };
    Getsy.prototype.showFrame = function () {
        if (!this.iframe)
            throw Error('No iframe to show.');
        this.iframe.style.opacity = '1';
        this.iframe.style.zIndex = '1000';
        this.iframe.style.pointerEvents = 'auto';
    };
    return Getsy;
}());
exports.Getsy = Getsy;
function getsy(url, options) {
    return new Promise(function (resolve, reject) {
        new Getsy(url, function (err, obj) {
            if (err)
                reject(err);
            resolve(obj);
        }, options);
    });
}
exports.default = getsy;
//# sourceMappingURL=getsy.js.map